/*****************************************************

Project : Meins
Version : 2.12
Date    : 10.07.2009
Author  : Bernhard (c) 2009
Company : Privat
Comments: Stark zusammengekrzte Version.


Chip type               : ATtiny24
AVR Core Clock frequency: 8,000000 MHz
*****************************************************/


#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

/*	Deffinitionen*/

#define Bit0    0x01
#define Bit1    0x02
#define Bit2    0x04
#define Bit3    0x08
#define Bit4    0x10
#define Bit5    0x20
#define Bit6    0x40
#define Bit7    0x80

#define Clear_Bit0    0xFE
#define Clear_Bit1    0xFD
#define Clear_Bit2    0xFB
#define Clear_Bit3    0xF7
#define Clear_Bit4    0xEF
#define Clear_Bit5    0xDF
#define Clear_Bit6    0xBF
#define Clear_Bit7    0x7F

#ifndef NULL
#define NULL    0x00
#endif

/*  Die Anzahl der maximal untersttzten Kanle des Empfngers */
#define DEF_MAX_INPUT       0x08
/*    (DEF_MAX_INPUT + 1) *2 */
#define DEF_MAX_SIGNAL_VAL   0x12

/*	Wenn folgende Signalwerte gendert werden, mu die Aktivierung
	der Failsave-Signalausgabe angepasst werden */
#define DEF_PORT_SIGNAL_1    PINA
#define DEF_PORT_SIGNAL_3    PINA
#define DEF_PORT_SIGNAL_5    PINA
#define DEF_PORT_SIGNAL_6    PINA  
#define DEF_PORT_SIGNAL_7    PINA

#define DEF_PIN_SIGNAL_0     Bit0
#define DEF_PIN_SIGNAL_1     Bit1
#define DEF_PIN_SIGNAL_2     Bit1
#define DEF_PIN_SIGNAL_3     Bit2
#define DEF_PIN_SIGNAL_4     Bit2
#define DEF_PIN_SIGNAL_5     Bit3
#define DEF_PIN_SIGNAL_6     Bit7
#define DEF_PIN_SIGNAL_7     Bit6

// Bitmaske um die Empfngereingnge invertieren zu knnen
#define DEF_BITMASKE_A      0xCE

// Speicher-Feldpositionen der einzelnen Kanle
#define DEF_FELD_SIGNAL_0     0
#define DEF_FELD_SIGNAL_1     2
#define DEF_FELD_SIGNAL_2     4
#define DEF_FELD_SIGNAL_3     6
#define DEF_FELD_SIGNAL_4     8
#define DEF_FELD_SIGNAL_5    10
#define DEF_FELD_SIGNAL_6    12
#define DEF_FELD_SIGNAL_7    14

//  Anvangswerte von Variablen 

#define DEF_SIGNAL_MINIMUM  1000
#define DEF_SIGNAL_MITTE    2500
#define DEF_SIGNAL_MAXIMUM  6000
#define DEF_DUMMY		       0


/*	Deffinitionen */
#define	Timer1An	TCCR1B=0x02
#define	Timer1Aus	TCCR1B=0x00


//  berprfen der Deffinitionen // 

#if (((DEF_MAX_INPUT+1)*2) != DEF_MAX_SIGNAL_VAL)
#error "Es ist ein Fehler bei der Eingabe von DEF_MAX_INPUT oder DEF_MAX_SIGNAL_VAL gemacht worden!"
#endif
 

/* Globale Variablen */


/*  Variable in die die Takte der Signallnge gespeichert werden 
    Die Reihen Folge der Daten ist:
    Signal 1     Zusatzdaten1
    Signal 2     Zusatzdaten2
    Signal 3     Zusatzdaten3
    Signal 4     Zusatzdaten4
    Signal 5     Zusatzdaten5
    Signal 6     Zusatzdaten6
    Signal 7     Zusatzdaten7
    Signal 8     Zusatzdaten8
    Checksumme   Zusatzdaten */
unsigned short  usInputSignal[DEF_MAX_SIGNAL_VAL] = {	DEF_SIGNAL_MINIMUM,	DEF_DUMMY,
																DEF_SIGNAL_MITTE,	DEF_DUMMY,
                                                       			DEF_SIGNAL_MITTE,	DEF_DUMMY,
																DEF_SIGNAL_MITTE,	DEF_DUMMY,
																DEF_SIGNAL_MAXIMUM,	DEF_DUMMY,
																DEF_SIGNAL_MITTE, 	DEF_DUMMY,
																DEF_SIGNAL_MITTE, 	DEF_DUMMY,
																DEF_SIGNAL_MINIMUM, DEF_DUMMY,
																NULL,				DEF_DUMMY};
/*  Variable in die die Takte der Signal-Startzeit gespeichert werden 
    Die Datenzuordnung ist die Gleiche wie bei usInputSignal */
unsigned short  usStartSignal[DEF_MAX_INPUT] ;
/*  DEF_BITMASKE_A / DEF_BITMASKE_B steht fr invertierende Signale, 0x00 fr nicht invertierende Signale */
unsigned char   uchBitMaskeXorA;
/* Sicherung des letzten Portwertes -> Wird fr die Ermittlung der Inderuptquelle bentigt */
unsigned char   uchPortAalt;
/* Variable zum Abspeichern binrer Statusanzeigen*/
unsigned char   uchGlobalFlags;


// Pin change 0-7 interrupt service routine
ISR( PCINT0_vect )
{
	// Sichern des Zhlerwertes, da der Zhler nicht angehalten werden 
	Timer1Aus;// Der Timer wird ber eine Deffinition angehalten
	unsigned short  usTimerTemp = TCNT1; 
	Timer1An;// Der Timer wird ber eine Deffinition gestartet

	// Sichern des Port's, da sich dieser whrend des Abarbeiten des Interupt's ndern kann
   	unsigned char   uchPortIn = PINA ^ uchBitMaskeXorA;

//////////////////// Signal 1 ////////////////////    
	// Ermitteln des aktuellen Signalwertes
	unsigned short uchWert	= uchPortIn & DEF_PIN_SIGNAL_1;	
	
	// Pegelnderung bei Signal1?
	if(uchWert != (uchPortAalt & DEF_PIN_SIGNAL_1))
	{
		/*High/Low Pegel?*/

		if(uchWert > 0)/* High-Pegel*/
		{
			usStartSignal[1]= usTimerTemp; // Sichern der Startzeit;
		
		}
		else	/* Low-Pegel*/
		{	
			// Berechnen der Signalzeit
			// Ein Auftreten des berlaufs des 16 Bit Zhlers whrend der Zeitmessung
			// wird bewusst in Kauf genommen, da dadurch nur eine Abweichung von 1 auftritt
			usInputSignal[DEF_FELD_SIGNAL_1 ] = usTimerTemp - usStartSignal[1];

		}
		uchPortAalt = uchPortIn; // Aktueller Portwert = Portwert fr nchste fr die nchste Zeitermittlung
		return;
	}
//////////////////// Signal 3 ////////////////////    
	// Ermitteln des aktuellen Signalwertes
	uchWert	= uchPortIn & DEF_PIN_SIGNAL_3;	
	
	// Pegelnderung bei Signal3?
	if(uchWert != (uchPortAalt & DEF_PIN_SIGNAL_3))
	{
		/*High/Low Pegel?*/

		if(uchWert > 0)/* High-Pegel*/
		{
			usStartSignal[3]= usTimerTemp; // Sichern der Startzeit;
		}
		else	/* Low-Pegel*/
		{	
			// Berechnen der Signalzeit
			usInputSignal[DEF_FELD_SIGNAL_3 ] = usTimerTemp - usStartSignal[3];
		}
		uchPortAalt = uchPortIn; // Aktueller Portwert = Portwert fr nchste fr die nchste Zeitermittlung
		return;
	}
//////////////////// Signal 5 ////////////////////    
	// Ermitteln des aktuellen Signalwertes
	uchWert	= uchPortIn & DEF_PIN_SIGNAL_5;	
	
	// Pegelnderung bei Signal5?
	if(uchWert != (uchPortAalt & DEF_PIN_SIGNAL_5))
	{
		/*High/Low Pegel?*/

		if(uchWert > 0)/* High-Pegel*/
		{
			usStartSignal[5]= usTimerTemp; // Sichern der Startzeit;
		}
		else	/* Low-Pegel*/
		{	
			// Berechnen der Signalzeit
			usInputSignal[DEF_FELD_SIGNAL_5 ] = usTimerTemp - usStartSignal[5];	
		}
		uchPortAalt = uchPortIn; // Aktueller Portwert = Portwert fr nchste fr die nchste Zeitermittlung
		return;
	}
//////////////////// Signal 6 ////////////////////    
	// Ermitteln des aktuellen Signalwertes
	uchWert	= uchPortIn & DEF_PIN_SIGNAL_6;	
	
	// Pegelnderung bei Signal6?
	if(uchWert != (uchPortAalt & DEF_PIN_SIGNAL_6))
	{
		/*High/Low Pegel?*/

		if(uchWert > 0)/* High-Pegel*/
		{
			usStartSignal[6]= usTimerTemp; // Sichern der Startzeit;
		}
		else	/* Low-Pegel*/
		{	
			// Berechnen der Signalzeit
			usInputSignal[DEF_FELD_SIGNAL_6 ] = usTimerTemp - usStartSignal[6];
		}
		uchPortAalt = uchPortIn; // Aktueller Portwert = Portwert fr nchste fr die nchste Zeitermittlung
		return;
	}
//////////////////// Signal 7 ////////////////////    
	// Ermitteln des aktuellen Signalwertes
	uchWert	= uchPortIn & DEF_PIN_SIGNAL_7;	
	
	// Pegelnderung bei Signal7?
	if(uchWert != (uchPortAalt & DEF_PIN_SIGNAL_7))
	{
		/*High/Low Pegel?*/

		if(uchWert > 0)/* High-Pegel*/
		{
			usStartSignal[7]= usTimerTemp; // Sichern der Startzeit;
		}
		else	/* Low-Pegel*/
		{	
			// Berechnen der Signalzeit

			usInputSignal[DEF_FELD_SIGNAL_7 ] = usTimerTemp - usStartSignal[7];
		}
		uchPortAalt = uchPortIn; // Aktueller Portwert = Portwert fr nchste fr die nchste Zeitermittlung
		return;
	}
}

/*  Funktion zum Initialisieren des Prozessors
    Diese Funktion wird von der Main-Schleife aufgerufen */

/*  Funktion zum Initialisieren der Ein- und Ausgnge */
void InitEA(void)
{
	// Input/Output Ports initialization
	// Port A initialization
	// Func7=In Func6=In Func5=Out Func4=Out Func3=In Func2=In Func1=In Func0=Out 
	// State7=P State6=P State5=0 State4=0 State3=P State2=P State1=P State0=0 
	PORTA=0xCE;
	DDRA=0x31;
    // Port B initialization
    // Func3=In Func2=In Func1=In Func0=In 
    // State3=T State2=P State1=P State0=P 
    PORTB=0x07;
    DDRB=0x00;
}

void InitTimer_Interrupt(void)
{
 // Timer0 Aus
	TCCR0A=0x00;
	TCCR0B=0x00;
	TCNT0=0x00;
	OCR0A=0x00;
	OCR0B=0x00;
		
	// Timer/Counter 1 initialization
	// Clock source: System Clock
	// Clock value: 1000,000 kHz
	// Mode: Normal top=FFFFh
	// OC1A output: Discon.
	// OC1B output: Discon.
	// Noise Canceler: Off
	// Input Capture on Falling Edge
	// Timer 1 Overflow Interrupt: Off
	// Input Capture Interrupt: Off
	// Compare A Match Interrupt: Off
	// Compare B Match Interrupt: Off
	TCCR1A=0x00;
	//TCCR1B=0x02;
	Timer1An;
	TCNT1H=0x00;
	TCNT1L=0x00;
	ICR1H=0x00;
	ICR1L=0x00;
	OCR1AH=0x00;
	OCR1AL=0x00;
	OCR1BH=0x00;
	OCR1BL=0x00;


	// External Interrupt(s) initialization
	// INT0: Off
	// Interrupt on any change on pins PCINT0-7: On
	// Interrupt on any change on pins PCINT8-11: Off
	MCUCR=0x00;
	PCMSK0=0xFE;
	GIMSK=0x10;
	GIFR=0x10;

	// Timer/Counter 0 Interrupt(s) initialization
	TIMSK0=0x00;
	// Timer/Counter 1 Interrupt(s) initialization
	TIMSK1=0x00;

}

void InitInterface(void)
{
    // Universal Serial Interface initialization
    // Mode: Disabled
    // Clock source: Register & Counter=no clk.
    // USI Counter Overflow Interrupt: Off
    USICR=0x00;
}

void InitADC(void)
{
    // Analog Comparator initialization
    // Analog Comparator: Off
    // Analog Comparator Input Capture by Timer/Counter 1: Off
    ACSR=0x80;
    ADCSRB=0x00;
}

void Init(void)
{
    // Crystal Oscillator division factor: 1  
    // -> Interner Takt = 8MHz
    CLKPR=0x80;
    CLKPR=0x00;
                                                      
    InitEA();  
    InitTimer_Interrupt();
    InitInterface();
    InitADC(); 
}
// Declare your global variables here

int main(void)
{
	Init();
	// Global enable interrupts
	uchBitMaskeXorA = 0;
	sei();

	while (1)
    {

	};
	return 0;
}

